Package: rocTools
Type: Package
Title: Tools for Distribution-Based ROC Smoothing Using 'pROC'
Version: 0.1.3
Authors@R: person(
  given="Grant C.",
  family="O'Connell",
  role = c("aut", "cre"),
  email="goconnell.phd@gmail.com",
  comment=c(ORCID = "0000-0002-2642-6670"))
Description: Extends the functionality of the 'pROC' package for conducting smoothed receiver
  operating characteristic (ROC) curve analysis. Enables automated selection of the distribution
  families to be fit when smoothing ROC curves via the population probability density function
  estimation strategy described by Leeflang et al. (2008) <doi:10.1373/clinchem.2007.096032>,
  as well as generation of diagnostic performance and cutoff estimates from the resultant
  smoothed curves.
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Imports: MASS, pROC (>= 1.19.0)
Suggests: testthat (>= 3.0.0)
Author: Grant C. O'Connell [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2642-6670>)
Maintainer: Grant C. O'Connell <goconnell.phd@gmail.com>
Repository: CRAN
Depends: R (>= 3.5)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-04 15:00:56 UTC; gco6
Date/Publication: 2026-05-07 16:10:13 UTC
Built: R 4.5.2; ; 2026-05-07 21:02:42 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print, roc.fits)
export(find.distr)
export(get.fits)
export(sm.coords)
import(pROC)
importFrom(MASS,fitdistr)
importFrom(stats,optimize)
importFrom(stats,pgamma)
importFrom(stats,plnorm)
importFrom(stats,pnorm)
importFrom(stats,pweibull)
importFrom(stats,qgamma)
importFrom(stats,qlnorm)
importFrom(stats,qnorm)
importFrom(stats,qweibull)
importFrom(stats,setNames)
importFrom(stats,uniroot)

#' Return distribution fits for smoothed ROC curves.
#'
#' Retrieves the  distribution fits associated with smoothed ROC curves generated using the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package via the` “fitdist” ` method. A `smooth.roc` object is provided as input, and the fitted case and control distribution families, parameters, and AIC values are returned. Can be used downstream of [find.distr()] and \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to inspect and evaluate the final distribution fits ultimately used to generate the smoothed ROC curve.
#'
#' @param smooth.roc
#' A `smooth.roc` object generated by \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} using `method = "fitdistr"`.
#'
#' @return
#' A nested list with the following elements:
#' \item{`controls`}{A list containing fit information for the fitted control distribution:
#'   \describe{
#'     \item{`distribution`}{The distribution family.}
#'     \item{`parameters`}{A named numeric vector containing the distribution parameters.}
#'     \item{`aic`}{The AIC value.}
#'   }
#' }
#' \item{`cases`}{A list containing fit information for the fitted case distribution:
#'   \describe{
#'     \item{`distribution`}{The distribution family.}
#'     \item{`parameters`}{A named numeric vector containing the distribution parameters.}
#'     \item{`aic`}{The AIC value.}
#'   }
#' }
#'
#' Objects returned by [get.fits()] contain a custom print method that displays a human readable summary of the fitted distributions when printed at the console.
#'
#' @seealso
#' \ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
#' \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
#' \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
#' [rocTools-package] \cr
#' [find.distr()] \cr
#' \code{\link{blood_data}}
#'
#' @examples
#' # load packages and example blood biomarker data
#'
#' library(pROC)
#' library(rocTools)
#' data(blood_data)
#'
#' # build raw ROC curve
#'
#' raw_roc <- roc(
#'   blood_data,
#'   response = "Diagnosis",
#'   predictor = "NfL"
#' )
#'
#' # find best fitting case and control distribution
#' # families to use for ROC smoothing
#'
#' best_fits <- find.distr(raw_roc)
#'
#' # use best fitting families to build smooth ROC curve
#'
#' smooth_roc <- smooth(
#'   raw_roc,
#'   method = "fitdistr",
#'   density.controls = best_fits$density.controls,
#'   density.cases = best_fits$density.cases,
#'   start.controls = best_fits$start.controls,
#'   start.cases = best_fits$start.cases
#' )
#'
#' # print final distribution fits used
#' # to build the smoothed curve
#'
#' get.fits(smooth_roc)
#'
#' @export

get.fits<-function(smooth.roc){

  smooth_roc<-smooth.roc

  if(!inherits(smooth_roc, "smooth.roc")){
    stop("Input must be a smooth.roc object generated by the 'pROC' package.")
  }

  if(is.null(smooth_roc$smoothing.args$method)||
     smooth_roc$smoothing.args$method!="fitdistr"){
    stop("Input must be a smooth.roc object generated with method = \"fitdistr\".")
  }

  extract_one<-function(fit){

    if(inherits(fit, "try-error")||is.null(fit)){
      stop("Distribution fitting failed; cannot extract parameters.")
    }

    dist<-fit$densfun
    params<-fit$estimate

    if(!is.numeric(params)||length(params)<1){
      stop("Invalid parameter estimates in smooth.roc object.")
    }

    k<-length(params)
    ll<-fit$loglik

    if(!is.finite(ll)){
      stop("Non-finite log-likelihood in fitted distribution.")
    }

    aic<-2*k-2*ll

    list(
      distribution=dist,
      parameters=params,
      aic=aic
    )
  }

  out<-list(
    controls=extract_one(smooth_roc$fit.controls),
    cases=extract_one(smooth_roc$fit.cases)
  )

  class(out)<-"roc.fits"

  return(out)
}

test_that("get.fits extracts fitted distributions from smooth.roc object",{

  skip_if_not_installed("pROC")
  library(pROC)

  data(blood_data)

  roc_obj<-roc(
    blood_data,
    response="Diagnosis",
    predictor="NfL",
    quiet=TRUE
  )

  fits<-find.distr(roc_obj, quiet=TRUE)

  smooth_roc<-do.call(
    smooth,
    c(list(roc_obj, method="fitdistr"), fits)
  )

  extracted<-get.fits(smooth_roc)

  expect_s3_class(extracted, "roc.fits")
  expect_named(extracted, c("controls", "cases"))

  expect_named(
    extracted$controls,
    c("distribution", "parameters", "aic")
  )

  expect_named(
    extracted$cases,
    c("distribution", "parameters", "aic")
  )

  expect_true(is.numeric(extracted$controls$aic))
  expect_true(is.numeric(extracted$cases$aic))
})

#' Automatically select distribution families to be used for ROC smoothing.
#'
#' Automatically selects the best fitting distribution families to be used when smoothing ROC curves via population probability density function estimation. It takes a raw `roc` object generated by the \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} function of the ‘pROC’ package as input, fits up to four different candidate distribution families to both the case and control data, and then uses AIC to select and return the best fitting distribution types along with any associated start values. These returned parameters can then be directly passed to the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package for generation of a smoothed ROC curve that approximates population-level diagnostic performance using the `"fitdistr"` method.
#'
#' Similar to \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}}, all fits are internally performed using the \ifelse{html}{\code{\link[MASS:fitdistr]{fitdistr()}}}{\code{fitdistr()}} function of the ‘MASS’ package.  Method of moments is used to establish starting values for distribution types where they are explicitly needed. When using [find.distr()] in tandem with \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}}, [get.fits()] can be used to retrieve the final fits ultimate used by \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} for generation of the smooth ROC curve. Furthermore, [sm.coords()] can be used to calculate population-level diagnostic performance and cutoff estimates from the smoothed curve.
#'
#' @param roc
#' A `roc` object generated by the \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} function.
#'
#' @param candidates
#' A character vector specifying the candidate distribution families to be tested. Options include ` “normal” `, ` “lognormal” `, ` “gamma” `, and ` “weibull” `. If ` NULL `, all four families will be tested by default.
#'
#' @param search.mode
#' Character string specifying what search mode to use to select the best fitting distribution families. One of `"all_pairs"` or `"per_group"`. If `"all_pairs"`, all possible case and control distribution family combinations are tested jointly, and the pair that minimizes the combined AIC across both groups is selected and returned. If `"per_group"`, distribution families are tested independently for the case and control groups, and the distribution families that respectively minimize AIC within each group are selected and returned. Default is `"all_pairs"`. If  `"all_pairs"`mode fails, `"per_group"` mode is used as a fallback with warning message.
#'
#' @param quiet
#' Specifies whether to silence messages generated during distribution selection. Default value is `TRUE`.
#'
#' @return
#' A named list containing the parameters associated with the best fitting case and control distribution families. The returned list can be passed directly to \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} for generation of a smooth ROC curve using the `"fitdistr"` method, and has the following elements:
#'
#' \item{`density.controls`}{A character string specifying the distribution family selected for the control group.}
#' \item{`density.cases`}{A character string specifying the distribution family selected for the case group.}
#' \item{`start.controls`}{An optional named list of starting values for fitting the control distribution. Returned only for distribution families that require explicit starting values.}
#' \item{`start.cases`}{An optional named list of starting values for fitting the case distribution. Returned only for distribution families that require explicit starting values.}
#'
#' Diagnostic information associated with model selection process is attached attributes, including all calculated AIC values and the final search mode.
#'
#' @seealso
#' \ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
#' \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
#' \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
#' \ifelse{html}{\code{\link[MASS:fitdistr]{fitdistr()}}}{\code{MASS::fitdistr()}} \cr
#' [rocTools-package] \cr
#' [get.fits()] \cr
#' [sm.coords()] \cr
#' \code{\link{blood_data}}
#'
#' @examples
#' # EXAMPLE USING LONG CALLS
#'
#' # load packages and example blood biomarker data
#'
#' library(pROC)
#' library(rocTools)
#' data(blood_data)
#'
#' # build raw ROC curve
#'
#' raw_roc <- roc(
#'   blood_data,
#'   response = "Diagnosis",
#'   predictor = "NfL"
#' )
#'
#' # find best fitting case and control distribution
#' # families to use for ROC smoothing
#'
#' best_fits <- find.distr(
#'   raw_roc,
#'   candidates = c("normal", "gamma", "lognormal", "weibull"),
#'   search.mode = "all_pairs",
#'   quiet = FALSE
#' )
#'
#' # use best fitting families to build smooth ROC curve
#'
#' smooth_roc <- smooth(
#'   raw_roc,
#'   method = "fitdistr",
#'   density.controls = best_fits$density.controls,
#'   density.cases = best_fits$density.cases,
#'   start.controls = best_fits$start.controls,
#'   start.cases = best_fits$start.cases
#' )
#'
#' # EXAMPLE USING COMPACT CALLS
#'
#' # load packages and example blood biomarker data
#'
#' library(pROC)
#' library(rocTools)
#' data(blood_data)
#'
#' # build raw ROC curve
#'
#' raw_roc <- roc(blood_data$Diagnosis, blood_data$NfL)
#'
#' # find best fitting case and controls distribution families
#' # and build smooth ROC curve in single call
#'
#' smooth_roc <- do.call(
#'   smooth,
#'   c(list(raw_roc, method = "fitdistr"), find.distr(raw_roc))
#' )
#'
#' @importFrom stats optimize pgamma plnorm pnorm pweibull qgamma qlnorm qnorm qweibull setNames uniroot
#' @importFrom MASS fitdistr
#' @import pROC
#' @export

find.distr<-function(
    roc,
    candidates=c("normal", "gamma", "lognormal", "weibull"),
    search.mode=c("all_pairs", "per_group"),
    quiet=FALSE
){

  # parse and check input
  # -----------------------------------------------------------

  # check if roc object

  roc_obj<-roc

  if(!inherits(roc_obj, "roc")){
    stop("Input must be a roc object generated by the 'pROC' package.")
  }

  allowed<-c("normal", "gamma", "lognormal", "weibull")
  if(!all(candidates%in%allowed)){
    stop(
      "Invalid candidate distribution(s). Allowed distributions include: ",
      paste(allowed, collapse=", ")
    )
  }

  candidates<-unique(candidates)
  search.mode<-match.arg(search.mode)

  controls<-roc_obj$controls
  cases<-roc_obj$cases

  # check for zero values

  if(any(controls<=0)||any(cases<=0)){

    candidates<-setdiff(candidates, "lognormal")

    if(quiet==FALSE){
      message(
        "Excluding lognormal: requires strictly positive values in both groups."
      )
    }
  }

  if(length(candidates)==0){
    stop("No candidate distributions remain after excluding lognormal.")
  }


  # shift x if any zero values are present

  make_positive<-function(x){

    if(any(x<=0)){

      xp<-x[x>0]

      if(length(xp)==0){
        shift<-1
      }else{
        shift<-min(xp)/1000
      }

      x<-x+shift
    }

    x
  }

  # build AIC helper
  # -----------------------------------------------------------

  safe_aic<-function(fit){

    if(inherits(fit, "try-error")){
      return(Inf)
    }

    ll<-fit$loglik

    if(!is.finite(ll)){
      return(Inf)
    }

    k<-length(fit$estimate)
    2*k-2*ll
  }

  # establish start values
  # -----------------------------------------------------------

  gamma_starts<-function(x){

    x<-make_positive(x)
    m<-mean(x)
    v<-var(x)

    if(!is.finite(m)||!is.finite(v)||m<=0||v<=0){
      return(list(shape=1, rate=1))
    }

    list(
      shape=max(m^2/v, 0.1),
      rate=max(m/v, 1e-4)
    )
  }

  weibull_mom_starts<-function(x){

    x<-make_positive(x)
    m<-mean(x)
    v<-var(x)

    if(!is.finite(m)||!is.finite(v)||m<=0||v<=0){
      return(list(shape=1.5, scale=max(m, 0.1)))
    }

    cv<-sqrt(v)/m

    moment_eq<-function(k){
      g1<-gamma(1+1/k)
      g2<-gamma(1+2/k)
      sqrt(g2/g1^2-1)-cv
    }

    root<-try(uniroot(moment_eq, c(0.1, 10)), silent=TRUE)

    if(inherits(root, "try-error")){
      k<-1.5
    }else{
      k<-root$root
    }

    lambda<-m/gamma(1+1/k)

    list(
      shape=max(k, 0.1),
      scale=max(lambda, 0.1)
    )
  }

  fit_one<-function(x, dist_name){

    x2<-make_positive(x)

    starts<-switch(
      dist_name,
      gamma=gamma_starts(x),
      weibull=weibull_mom_starts(x),
      NULL
    )

    suppressWarnings(
      try(
        MASS::fitdistr(x2, dist_name, start=starts),
        silent=TRUE
      )
    )
  }

  # calculate per-group AIC
  # -----------------------------------------------------------

  fits_ctrl<-lapply(candidates, fit_one, x=controls)
  fits_case<-lapply(candidates, fit_one, x=cases)

  AIC_ctrl<-setNames(sapply(fits_ctrl, safe_aic), candidates)
  AIC_case<-setNames(sapply(fits_case, safe_aic), candidates)

  if(all(!is.finite(AIC_ctrl))){
    stop("All candidate distributions failed to fit controls.")
  }

  if(all(!is.finite(AIC_case))){
    stop("All candidate distributions failed to fit cases.")
  }

  # select best fits
  # -----------------------------------------------------------

  if(search.mode=="per_group"){

    if(quiet==FALSE){
      message(
        "Per-group AIC selection across: ",
        paste(candidates, collapse=", ")
      )
    }

    best_ctrl<-candidates[which.min(AIC_ctrl)]
    best_case<-candidates[which.min(AIC_case)]

    AIC_pairs<-NA_real_

  }else{

    pairs<-expand.grid(
      ctrl=candidates,
      case=candidates,
      stringsAsFactors=FALSE
    )

    ic_vals<-vapply(
      seq_len(nrow(pairs)),
      function(i){
        AIC_ctrl[pairs$ctrl[i]]+AIC_case[pairs$case[i]]
      },
      numeric(1)
    )

    names(ic_vals)<-paste(pairs$ctrl, pairs$case, sep="_")
    best_i<-which.min(ic_vals)

    if(!length(best_i)||!is.finite(ic_vals[best_i])){

      if(quiet==FALSE){
        warning(
          "All-pairs AIC selection failed; falling back to per-group."
        )
      }

      best_ctrl<-candidates[which.min(AIC_ctrl)]
      best_case<-candidates[which.min(AIC_case)]
      AIC_pairs<-NA_real_

    }else{

      best_ctrl<-pairs$ctrl[best_i]
      best_case<-pairs$case[best_i]
      AIC_pairs<-ic_vals
    }

    if(quiet==FALSE){
      message(
        "All-pairs AIC across: ",
        paste(candidates, collapse=", ")
      )
      message(
        "Selected: controls = ", best_ctrl,
        ", cases = ", best_case
      )
    }
  }

  # build output
  # -----------------------------------------------------------

  out<-list(
    density.controls=best_ctrl,
    density.cases=best_case,
    start.controls=switch(
      best_ctrl,
      gamma=gamma_starts(controls),
      weibull=weibull_mom_starts(controls),
      NULL
    ),
    start.cases=switch(
      best_case,
      gamma=gamma_starts(cases),
      weibull=weibull_mom_starts(cases),
      NULL
    )
  )

  # attach attributes
  # -----------------------------------------------------------

  attr(out, "aic")<-list(
    controls=AIC_ctrl,
    cases=AIC_case,
    all_pairs=AIC_pairs
  )
  attr(out, "search.mode")<-search.mode

  return(out)
}

test_that("sm.coords returns valid dx performance and cutoff estimates",{

  skip_if_not_installed("pROC")
  library(pROC)

  data(blood_data)

  roc_obj<-roc(
    blood_data,
    response="Diagnosis",
    predictor="NfL",
    quiet=TRUE
  )

  fits<-find.distr(roc_obj, quiet=TRUE)

  smooth_roc<-do.call(
    smooth,
    c(list(roc_obj, method="fitdistr"), fits)
  )

  res<-sm.coords(smooth_roc, type="best", quiet=TRUE)

  expect_type(res, "double")
  expect_named(res, c("threshold", "specificity", "sensitivity"))

  expect_true(res["specificity"]>=0&&res["specificity"]<=1)
  expect_true(res["sensitivity"]>=0&&res["sensitivity"]<=1)
})

test_that("sm.coords errors on invalid input",{

  expect_error(
    sm.coords("not a smooth.roc"),
    "smooth.roc"
  )
})

test_that("find.distr returns valid distribution parameters",{

  skip_if_not_installed("pROC")
  library(pROC)

  data(blood_data)

  roc_obj<-roc(
    blood_data,
    response="Diagnosis",
    predictor="NfL",
    quiet=TRUE
  )

  fits<-find.distr(roc_obj, quiet=TRUE)

  expect_type(fits, "list")
  expect_named(
    fits,
    c("density.controls", "density.cases", "start.controls", "start.cases")
  )

  expect_true(is.character(fits$density.controls))
  expect_true(is.character(fits$density.cases))

  expect_true(
    fits$density.controls%in%c("normal", "gamma", "weibull", "lognormal")
  )

  expect_true(
    fits$density.cases%in%c("normal", "gamma", "weibull", "lognormal")
  )
})

test_that("smoothed ROC curve can be generated using find.distr output",{

  skip_if_not_installed("pROC")
  library(pROC)

  data(blood_data)

  roc_obj<-roc(
    blood_data,
    response="Diagnosis",
    predictor="NfL",
    quiet=TRUE
  )

  fits<-find.distr(roc_obj, quiet=TRUE)

  smooth_roc<-do.call(
    smooth,
    c(list(roc_obj, method="fitdistr"), fits)
  )

  expect_s3_class(smooth_roc, "smooth.roc")
})

#' Calculate diagnostic performance and cutoff estimates from smoothed ROC curves.
#'
#' Calculates population-level sensitivity, specificity, and diagnostic cutoff estimates from smoothed ROC curves generated using the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package via the ` “fitdist” ` method. A `smooth.roc` object is provided as input, and metrics can be calculated for the estimated diagnostic cutoff that maximizes the Youden index, or for the estimated diagnostic cutoff that achieves a user specified sensitivity or specificity value.
#'
#' @param smooth.roc
#' A `smooth.roc` object generated by \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} using `method = "fitdistr"`.
#'
#' @param type
#' Character string specifying the criterion used to define the diagnostic cutoff to return estimates for. If `"best"`, estimates for the cutoff that maximizes the Youden index are calculated and returned. If `"specificity"`, estimates for the cutoff that achieves a target specificity value specified by `target.value` are calculated and returned. If `"sensitivity"`, estimates for the cutoff that achieves a target sensitivity value specified by `target.value` are calculated and returned.
#'
#' @param target.value
#' Numeric value ranging between 0 and 1 that specifies the target sensitivity or specificity value defining the cutoff to calculate estimates for when `type` is not `"best"`.
#'
#' @param method
#' Character string specifying the methodology used to generate estimates. If `"continuous"`, values are estimated via continuous optimization of the smoothed ROC curve using the fitted case and control population probability density functions. If `"grid"`, cutoff values are selected via optimization over the grid-discretized operating points of the smoothed ROC curve, matching the internal behavior of the native \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{coords()}}  function of the ‘pROC’ package. `"continuous"` provides finer resolution and potentially more accurate estimates; however, sensitivity and specificity values will not perfectly match those returned by the \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{coords()}}  function. `"continuous"` is default.
#'
#' @param quiet
#' Specifies whether to silence messages generated during estimate calculation. Default value is `FALSE`.
#'
#' @return
#' A named numeric vector containing the estimated diagnostic performance and cutoff values:
#'
#' \item{`threshold`}{ The estimated diagnostic cutoff value.}
#' \item{`specificity`}{The estimated specificity at the cutoff value.}
#' \item{`sensitivity`}{The estimated sensitivity at the cutoff value.}
#'
#' @seealso
#' \ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
#' \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
#' \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
#' \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{pROC::coords()}} \cr
#' [rocTools-package] \cr
#' [find.distr()] \cr
#' \code{\link{blood_data}}
#'
#' @examples
#' # load packages and example blood biomarker data
#'
#' library(pROC)
#' library(rocTools)
#' data(blood_data)
#'
#' # build raw ROC curve
#'
#' raw_roc <- roc(
#'   blood_data,
#'   response = "Diagnosis",
#'   predictor = "NfL"
#' )
#'
#' # find best fitting case and control distribution
#' # families to use for ROC smoothing
#'
#' best_fits <- find.distr(raw_roc)
#'
#' # use best fitting families to build smooth ROC curve
#'
#' smooth_roc <- smooth(
#'   raw_roc,
#'   method = "fitdistr",
#'   density.controls = best_fits$density.controls,
#'   density.cases = best_fits$density.cases,
#'   start.controls = best_fits$start.controls,
#'   start.cases = best_fits$start.cases
#' )
#'
#' # calculate sensitivity, specificity, and cutoff
#' # estimates for the max Youden cutoff
#'
#' sm.coords(smooth_roc, type = "best")
#'
#' # calculate sensitivity, specificity, and cutoff estimates
#' # for the cutoff that yields 90% sensitivity
#'
#' sm.coords(
#'   smooth_roc,
#'   type = "sensitivity",
#'   target.value = 0.9
#' )
#'
#' # calculate sensitivity, specificity, and cutoff estimates
#' # for the cutoff that yields 75% specificity
#'
#' sm.coords(
#'   smooth_roc,
#'   type = "specificity",
#'   target.value = 0.75
#' )
#'
#' @export

sm.coords<-function(
    smooth.roc,
    type=c("best", "specificity", "sensitivity"),
    target.value=NULL,
    method=c("continuous", "grid"),
    quiet=FALSE
){

  # check input
  # -----------------------------------------------------------

  smooth_roc<-smooth.roc
  method<-match.arg(method)
  type<-match.arg(type)

  if(!inherits(smooth_roc, "smooth.roc")){
    stop("Input must be a smooth.roc object generated by the 'pROC' package.")
  }

  if(is.null(smooth_roc$smoothing.args$method)||
     smooth_roc$smoothing.args$method!="fitdistr"){
    stop("Input must be a smooth.roc object generated with method = \"fitdistr\".")
  }

  roc_emp<-attr(smooth_roc, "roc")
  if(is.null(roc_emp)){
    stop(
      "Input must be a smooth.roc object with the original raw roc object ",
      "attached as an attribute."
    )
  }

  if(type!="best"){

    if(is.null(target.value)){
      stop("target.value must be supplied when type is not 'best'.")
    }

    if(!is.numeric(target.value)||
       length(target.value)!=1||
       target.value<=0||
       target.value>=1){
      stop("target.value must be a single numeric value between 0 and 1.")
    }
  }

  # extract case and control PDFs
  # -----------------------------------------------------------

  fit0<-smooth_roc$fit.controls
  fit1<-smooth_roc$fit.cases

  densfun_to_p<-function(d){
    switch(d,
           normal=pnorm,
           gamma=pgamma,
           lognormal=plnorm,
           weibull=pweibull,
           stop("Unsupported distribution: ", d)
    )
  }

  pfun0<-densfun_to_p(fit0$densfun)
  pfun1<-densfun_to_p(fit1$densfun)

  F0<-function(c){
    do.call(pfun0, c(list(c), as.list(fit0$estimate)))
  }

  F1<-function(c){
    do.call(pfun1, c(list(c), as.list(fit1$estimate)))
  }

  # helpers for calculating Youden index,
  # sensitivity, and specificity
  # -----------------------------------------------------------

  if(smooth_roc$direction=="<"){

    youden_c<-function(c){
      y<-F0(c)-F1(c)
      y[!is.finite(y)]<-0
      y
    }

    sens_c<-function(c){
      1-F1(c)
    }

    spec_c<-function(c){
      F0(c)
    }

  }else{

    youden_c<-function(c){
      y<-F1(c)-F0(c)
      y[!is.finite(y)]<-0
      y
    }

    sens_c<-function(c){
      F1(c)
    }

    spec_c<-function(c){
      1-F0(c)
    }
  }

  # extract raw x data
  # -----------------------------------------------------------

  x<-roc_emp$original.predictor
  lo<-min(x, na.rm=TRUE)
  hi<-max(x, na.rm=TRUE)
  eps<-1e-8*(hi-lo)

  # implement grid method
  # -----------------------------------------------------------

  if(method=="grid"){

    sens<-smooth_roc$sensitivities
    spec<-smooth_roc$specificities

    if(type=="best"){

      youden<-sens+spec-1
      idx<-which.max(youden)

    }else if(type=="sensitivity"){

      idx<-which.min(abs(sens-target.value))

      if(target.value<min(sens)||target.value>max(sens)){
        if(quiet==FALSE){
          warning(
            "Requested sensitivity not attainable: ",
            "Returning threshold for closest achievable sensitivity."
          )
        }
      }

    }else{

      idx<-which.min(abs(spec-target.value))

      if(target.value<min(spec)||target.value>max(spec)){
        if(quiet==FALSE){
          warning(
            "Requested specificity not attainable: ",
            "Returning threshold for closest achievable specificity."
          )
        }
      }
    }

    # recover fitted distribution cutoff from grid

    if(smooth_roc$direction=="<"){

      qfun0<-switch(fit0$densfun,
                    normal=qnorm,
                    gamma=qgamma,
                    lognormal=qlnorm,
                    weibull=qweibull)

      c_opt<-do.call(
        qfun0,
        c(list(spec[idx]), as.list(fit0$estimate))
      )

    }else{

      qfun1<-switch(fit1$densfun,
                    normal=qnorm,
                    gamma=qgamma,
                    lognormal=qlnorm,
                    weibull=qweibull)

      c_opt<-do.call(
        qfun1,
        c(list(sens[idx]), as.list(fit1$estimate))
      )
    }

    if(!is.finite(c_opt)){

      c_opt<-ifelse(c_opt>0, hi, lo)
      c_opt<-max(min(c_opt, hi-eps), lo+eps)

      if(type=="best"){
        warning(
          "Maximum Youden index is 0: ",
          "Returning boundary threshold from the raw roc object."
        )
      }
    }

    # build output

    return(c(
      threshold=c_opt,
      specificity=spec[idx],
      sensitivity=sens[idx]
    ))
  }

  # implement continuous method
  # -----------------------------------------------------------

  if(type=="best"){

    rng<-range(x, na.rm=TRUE)

    opt<-try(
      optimize(youden_c, interval=rng, maximum=TRUE),
      silent=TRUE
    )

    if(inherits(opt, "try-error")){
      stop("Continuous optimization of Youden index failed.")
    }

    c_opt<-opt$maximum

  }else if(type=="sensitivity"){

    obj<-function(c){
      abs(sens_c(c)-target.value)
    }

    opt<-optimize(obj, interval=c(lo, hi))
    c_opt<-opt$minimum

    if(abs(sens_c(c_opt)-target.value)>1e-6){
      if(quiet==FALSE){
        warning(
          "Requested sensitivity not attainable: ",
          "Returning threshold for closest achievable sensitivity."
        )
      }
    }

  }else{

    obj<-function(c){
      abs(spec_c(c)-target.value)
    }

    opt<-optimize(obj, interval=c(lo, hi))
    c_opt<-opt$minimum

    if(abs(spec_c(c_opt)-target.value)>1e-6){
      if(quiet==FALSE){
        warning(
          "Requested specificity not attainable: ",
          "Returning threshold for closest achievable specificity."
        )
      }
    }
  }

  if(!is.finite(c_opt)){

    c_opt<-ifelse(c_opt>0, hi, lo)
    c_opt<-max(min(c_opt, hi-eps), lo+eps)

    if(type=="best"){
      if(quiet==FALSE){
        warning(
          "Maximum Youden index is 0: ",
          "Returning boundary threshold from the raw roc object."
        )
      }
    }
  }

  # build output

  c(
    threshold=c_opt,
    specificity=spec_c(c_opt),
    sensitivity=sens_c(c_opt)
  )
}

#' Print distribution fits retrieved from smoothed ROC curves
#'
#' @param x
#' A `roc.fits` object generated by [get.fits()].
#'
#' @param ... Further arguments passed to or from other methods.
#'
#' @return
#' A console printed summary including the fitted case and control distribution families, parameters, and AIC values.
#'
#' @rawNamespace S3method(print, roc.fits)
#' @noRd

print.roc.fits<-function(x,...){

  fit<-x

  cat("Fitted distributions for smoothed ROC curve\n\n")

  for(grp in c("controls", "cases")){
    cat(tools::toTitleCase(grp), ":\n", sep="")
    cat("  Distribution: ", fit[[grp]]$distribution, "\n", sep="")
    cat("  Parameters:\n")
    print(fit[[grp]]$parameters)
    cat("  AIC: ", signif(fit[[grp]]$aic, 4), "\n\n", sep="")
  }

  invisible(fit)
}

#' Tools for Distribution-Based ROC Smoothing Using ‘pROC’
#'
#' Receiver operating characteristic (ROC) analysis is widely used in studies of diagnostic performance to generate key metrics such as sensitivity and specificity estimates, as well as establish candidate diagnostic cutoffs. However, population sample measures have a finite number of possible diagnostic cutoffs, which can result in raw ROC curves that appear jagged or stepped. This stepping phenomenon can inflate diagnostic performance estimates and reduce the accuracy of diagnostic cutoff estimates, particularly in small to modestly sized samples (Linnet and Brandt, 1986). One strategy to reduce this bias and increase generalizability is to carry out ROC smoothing using population probability density function estimation. While this solution is robust when properly implemented, smoothing using poorly fit distributions can unintentionally increase bias (Leeflang et al., 2008). To that end, the ‘rocTools’ package provides a set of functions that extend the functionality of the ‘pROC’ package for carrying out population probability density function estimation-based ROC smoothing and associated analyses. These functions enable automated selection of the case and control distribution families to be fit for smoothing, as well as generation of population-level diagnostic performance and diagnostic cutoff estimates from the resultant smoothed curves.
#'
#' The ‘rocTools’ package contains three user facing functions that can be used in combination with the core functions of the ‘pROC’ package to build and analyze smoothed ROC curves via population probability density function estimation methods.
#'
#' The [find.distr()] function performs automated selection of the distribution families to be fit when generating smoothed ROC curves using the ` “fitdistr” ` method of the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package. It takes a raw `roc` object generated by the \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} function of the ‘pROC’ package as input, fits up to four different distribution families to both the case and control data, and then uses Akaike information criterion (AIC) to select and return the best fitting distribution types along with any associated start values. These returned parameters can then be directly passed to \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to be used for ROC smoothing.
#'
#' The [get.fits()] function retrieves and returns the final case and control distribution fits from `smooth.roc` objects generated using the ` “fitdistr” ` method of the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function.  It can be used downstream of [find.distr()] and \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to evaluate the final distribution fits used for ROC smoothing.  The returned values include the distribution types, the distribution parameters, and the associated AIC values. The extracted values can either be printed in console as a human readable summary or stored as a new object.
#'
#' The [sm.coords()] function calculates population-level sensitivity, specificity, and diagnostic cutoff estimates from smoothed ROC curves generated using the ` “fitdist” ` method of the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function. A `smooth.roc` object is provided as input, and these metrics can be calculated for the estimated diagnostic cutoff that maximizes the Youden index , or for the estimated diagnostic cutoff that achieves a user specified sensitivity or specificity value.  This function works analogously to the \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{coords()}} function of the ‘pROC’ package, but produces more precise sensitivity and specificity estimates for smoothed curves through continuous optimization as opposed to grid-based optimization, and has the added functionality of producing diagnostic cutoff estimates.
#'
#' From an implementation perspective, this collection of functions can be called in the following order to create a seamless workflow to generate population-level diagnostic performance and cutoff estimates from case-control population sample measures: \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} to build a raw ROC curve for the population sample data, [find.distr()] to select the best case and control distribution families to fit for roc smoothing, \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to build a smooth ROC curve approximating population-level diagnostic performance using said distribution families, and [sm.coords()] to calculate population-level sensitivity, specificity, and diagnostic cutoff estimates associated with said smooth curve. [get.fits()] can be used as needed to retrieve the final distribution fits generated as part of this workflow.
#'
#' A real-world dataset containing blood biomarker measures is also included and is used throughout the documentation examples. The \code{\link{blood_data}} dataset contains plasma neurofilament-light chain (NfL) levels measured in 33 neurologically normal controls and 14 patients diagnosed with ischemic stroke as described by O'Connell et al. (2020).
#'
#' @section Main functions:
#'
#' \describe{
#'   \item{[find.distr()]}{Automatically selects distribution families to be used for ROC smoothing.}
#'   \item{[get.fits()]}{Returns distribution fits for smoothed ROC curves.}
#'   \item{[sm.coords()]}{Calculates diagnostic performance and cutoff estimates from smoothed ROC curves.}
#' }
#'
#' @section Datasets:
#' \describe{
#'   \item{\code{\link{blood_data}}}{Example blood biomarker dataset.}
#' }
#'
#' @section Citation:
#' If you use the ‘rocTools’ package in published work, please cite:
#'
#' O'Connell, GC. (2026).
#' \emph{rocTools}.
#' R package version 0.1.3.
#' Available from \url{https://CRAN.R-project.org/package=rocTools}.
#'
#' @references
#' Linnet, K. & Brandt, E. (1986).
#' Assessing diagnostic tests once an optimal cutoff point has been selected.
#' \emph{Clinical Chemistry}, 32, 7.
#' \doi{10.1093/clinchem/32.7.1341}
#'
#' Leeflang, MMG., Moons, KGM., Reitsma, JB., & Zwinderman, AH. (2008).
#' Bias in Sensitivity and Specificity Caused by Data-Driven Selection of Optimal Cutoff Values: Mechanisms, Magnitude, and Solutions.
#' \emph{Clinical Chemistry}, 54, 4.
#' \doi{10.1373/clinchem.2007.096032}
#'
#' Robin, X., Turck, N., Hainard, A., Tiberti, N., Lisacek, F., Sanchez, JC., & Müller, M. (2011).
#' pROC: an open-source package for R and S+ to analyze and compare ROC curves.
#' \emph{BMC Bioinformatics}, 12, 77.
#' \doi{10.1186/1471-2105-12-7}
#'
#' O'Connell, GC., Alder, ML., Smothers, CG., Still, CH., Webel, AR., & Moore, S.M. (2020).
#' Diagnosis of ischemic stroke using circulating levels of brain-specific proteins measured via high-sensitivity digital ELISA.
#' \emph{Brain Research}, 15, 1739.
#' \doi{10.1016/j.brainres.2020.146861}
#'
#' @seealso
#' \ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
#' \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
#' \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
#' \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{pROC::coords()}} \cr
#' [find.distr()] \cr
#' [get.fits()] \cr
#' [sm.coords()] \cr
#' \code{\link{blood_data}}
#'
#' @name rocTools-package

NULL

#' Example blood biomarker dataset.
#'
#' A real-world dataset containing plasma neurofilament-light chain (NfL) levels measured in 33 neurologically normal controls and 14 patients diagnosed with ischemic stroke as described by O'Connell et al. (2020).
#'
#' @format
#' A data frame with one column containing stroke diagnoses and another column containing plasma neurofilament-light chain levels in pg/mL.
#'
#' @references
#' O'Connell, GC., Alder, ML., Smothers, CG., Still, CH., Webel, AR., & Moore, S.M. (2020).
#' Diagnosis of ischemic stroke using circulating levels of brain-specific proteins measured via high-sensitivity digital ELISA.
#' \emph{Brain Research}, 15, 1739.
#' \doi{10.1016/j.brainres.2020.146861}
#'
#' @examples
#' #load example dataset
#'
#' data(blood_data)
#'
#' #return dataset structure
#'
#' str(blood_data)
#'
#' @name blood_data
#' @docType data

NULL

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_distr.R
\name{find.distr}
\alias{find.distr}
\title{Automatically select distribution families to be used for ROC smoothing.}
\usage{
find.distr(
  roc,
  candidates = c("normal", "gamma", "lognormal", "weibull"),
  search.mode = c("all_pairs", "per_group"),
  quiet = FALSE
)
}
\arguments{
\item{roc}{A \code{roc} object generated by the \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} function.}

\item{candidates}{A character vector specifying the candidate distribution families to be tested. Options include \verb{“normal”}, \verb{“lognormal”}, \verb{“gamma”}, and \verb{“weibull”}. If \code{NULL}, all four families will be tested by default.}

\item{search.mode}{Character string specifying what search mode to use to select the best fitting distribution families. One of \code{"all_pairs"} or \code{"per_group"}. If \code{"all_pairs"}, all possible case and control distribution family combinations are tested jointly, and the pair that minimizes the combined AIC across both groups is selected and returned. If \code{"per_group"}, distribution families are tested independently for the case and control groups, and the distribution families that respectively minimize AIC within each group are selected and returned. Default is \code{"all_pairs"}. If  \code{"all_pairs"}mode fails, \code{"per_group"} mode is used as a fallback with warning message.}

\item{quiet}{Specifies whether to silence messages generated during distribution selection. Default value is \code{TRUE}.}
}
\value{
A named list containing the parameters associated with the best fitting case and control distribution families. The returned list can be passed directly to \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} for generation of a smooth ROC curve using the \code{"fitdistr"} method, and has the following elements:

\item{\code{density.controls}}{A character string specifying the distribution family selected for the control group.}
\item{\code{density.cases}}{A character string specifying the distribution family selected for the case group.}
\item{\code{start.controls}}{An optional named list of starting values for fitting the control distribution. Returned only for distribution families that require explicit starting values.}
\item{\code{start.cases}}{An optional named list of starting values for fitting the case distribution. Returned only for distribution families that require explicit starting values.}

Diagnostic information associated with model selection process is attached attributes, including all calculated AIC values and the final search mode.
}
\description{
Automatically selects the best fitting distribution families to be used when smoothing ROC curves via population probability density function estimation. It takes a raw \code{roc} object generated by the \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} function of the ‘pROC’ package as input, fits up to four different candidate distribution families to both the case and control data, and then uses AIC to select and return the best fitting distribution types along with any associated start values. These returned parameters can then be directly passed to the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package for generation of a smoothed ROC curve that approximates population-level diagnostic performance using the \code{"fitdistr"} method.
}
\details{
Similar to \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}}, all fits are internally performed using the \ifelse{html}{\code{\link[MASS:fitdistr]{fitdistr()}}}{\code{fitdistr()}} function of the ‘MASS’ package.  Method of moments is used to establish starting values for distribution types where they are explicitly needed. When using \code{\link[=find.distr]{find.distr()}} in tandem with \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}}, \code{\link[=get.fits]{get.fits()}} can be used to retrieve the final fits ultimate used by \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} for generation of the smooth ROC curve. Furthermore, \code{\link[=sm.coords]{sm.coords()}} can be used to calculate population-level diagnostic performance and cutoff estimates from the smoothed curve.
}
\examples{
# EXAMPLE USING LONG CALLS

# load packages and example blood biomarker data

library(pROC)
library(rocTools)
data(blood_data)

# build raw ROC curve

raw_roc <- roc(
  blood_data,
  response = "Diagnosis",
  predictor = "NfL"
)

# find best fitting case and control distribution
# families to use for ROC smoothing

best_fits <- find.distr(
  raw_roc,
  candidates = c("normal", "gamma", "lognormal", "weibull"),
  search.mode = "all_pairs",
  quiet = FALSE
)

# use best fitting families to build smooth ROC curve

smooth_roc <- smooth(
  raw_roc,
  method = "fitdistr",
  density.controls = best_fits$density.controls,
  density.cases = best_fits$density.cases,
  start.controls = best_fits$start.controls,
  start.cases = best_fits$start.cases
)

# EXAMPLE USING COMPACT CALLS

# load packages and example blood biomarker data

library(pROC)
library(rocTools)
data(blood_data)

# build raw ROC curve

raw_roc <- roc(blood_data$Diagnosis, blood_data$NfL)

# find best fitting case and controls distribution families
# and build smooth ROC curve in single call

smooth_roc <- do.call(
  smooth,
  c(list(raw_roc, method = "fitdistr"), find.distr(raw_roc))
)

}
\seealso{
\ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
\ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
\ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
\ifelse{html}{\code{\link[MASS:fitdistr]{fitdistr()}}}{\code{MASS::fitdistr()}} \cr
\link{rocTools-package} \cr
\code{\link[=get.fits]{get.fits()}} \cr
\code{\link[=sm.coords]{sm.coords()}} \cr
\code{\link{blood_data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rocTools-package.R
\name{rocTools-package}
\alias{rocTools-package}
\title{Tools for Distribution-Based ROC Smoothing Using ‘pROC’}
\description{
Receiver operating characteristic (ROC) analysis is widely used in studies of diagnostic performance to generate key metrics such as sensitivity and specificity estimates, as well as establish candidate diagnostic cutoffs. However, population sample measures have a finite number of possible diagnostic cutoffs, which can result in raw ROC curves that appear jagged or stepped. This stepping phenomenon can inflate diagnostic performance estimates and reduce the accuracy of diagnostic cutoff estimates, particularly in small to modestly sized samples (Linnet and Brandt, 1986). One strategy to reduce this bias and increase generalizability is to carry out ROC smoothing using population probability density function estimation. While this solution is robust when properly implemented, smoothing using poorly fit distributions can unintentionally increase bias (Leeflang et al., 2008). To that end, the ‘rocTools’ package provides a set of functions that extend the functionality of the ‘pROC’ package for carrying out population probability density function estimation-based ROC smoothing and associated analyses. These functions enable automated selection of the case and control distribution families to be fit for smoothing, as well as generation of population-level diagnostic performance and diagnostic cutoff estimates from the resultant smoothed curves.
}
\details{
The ‘rocTools’ package contains three user facing functions that can be used in combination with the core functions of the ‘pROC’ package to build and analyze smoothed ROC curves via population probability density function estimation methods.

The \code{\link[=find.distr]{find.distr()}} function performs automated selection of the distribution families to be fit when generating smoothed ROC curves using the \verb{“fitdistr”} method of the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package. It takes a raw \code{roc} object generated by the \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} function of the ‘pROC’ package as input, fits up to four different distribution families to both the case and control data, and then uses Akaike information criterion (AIC) to select and return the best fitting distribution types along with any associated start values. These returned parameters can then be directly passed to \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to be used for ROC smoothing.

The \code{\link[=get.fits]{get.fits()}} function retrieves and returns the final case and control distribution fits from \code{smooth.roc} objects generated using the \verb{“fitdistr”} method of the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function.  It can be used downstream of \code{\link[=find.distr]{find.distr()}} and \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to evaluate the final distribution fits used for ROC smoothing.  The returned values include the distribution types, the distribution parameters, and the associated AIC values. The extracted values can either be printed in console as a human readable summary or stored as a new object.

The \code{\link[=sm.coords]{sm.coords()}} function calculates population-level sensitivity, specificity, and diagnostic cutoff estimates from smoothed ROC curves generated using the \verb{“fitdist”} method of the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function. A \code{smooth.roc} object is provided as input, and these metrics can be calculated for the estimated diagnostic cutoff that maximizes the Youden index , or for the estimated diagnostic cutoff that achieves a user specified sensitivity or specificity value.  This function works analogously to the \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{coords()}} function of the ‘pROC’ package, but produces more precise sensitivity and specificity estimates for smoothed curves through continuous optimization as opposed to grid-based optimization, and has the added functionality of producing diagnostic cutoff estimates.

From an implementation perspective, this collection of functions can be called in the following order to create a seamless workflow to generate population-level diagnostic performance and cutoff estimates from case-control population sample measures: \ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{roc()}} to build a raw ROC curve for the population sample data, \code{\link[=find.distr]{find.distr()}} to select the best case and control distribution families to fit for roc smoothing, \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to build a smooth ROC curve approximating population-level diagnostic performance using said distribution families, and \code{\link[=sm.coords]{sm.coords()}} to calculate population-level sensitivity, specificity, and diagnostic cutoff estimates associated with said smooth curve. \code{\link[=get.fits]{get.fits()}} can be used as needed to retrieve the final distribution fits generated as part of this workflow.

A real-world dataset containing blood biomarker measures is also included and is used throughout the documentation examples. The \code{\link{blood_data}} dataset contains plasma neurofilament-light chain (NfL) levels measured in 33 neurologically normal controls and 14 patients diagnosed with ischemic stroke as described by O'Connell et al. (2020).
}
\section{Main functions}{


\describe{
\item{\code{\link[=find.distr]{find.distr()}}}{Automatically selects distribution families to be used for ROC smoothing.}
\item{\code{\link[=get.fits]{get.fits()}}}{Returns distribution fits for smoothed ROC curves.}
\item{\code{\link[=sm.coords]{sm.coords()}}}{Calculates diagnostic performance and cutoff estimates from smoothed ROC curves.}
}
}

\section{Datasets}{

\describe{
\item{\code{\link{blood_data}}}{Example blood biomarker dataset.}
}
}

\section{Citation}{

If you use the ‘rocTools’ package in published work, please cite:

O'Connell, GC. (2026).
\emph{rocTools}.
R package version 0.1.3.
Available from \url{https://CRAN.R-project.org/package=rocTools}.
}

\references{
Linnet, K. & Brandt, E. (1986).
Assessing diagnostic tests once an optimal cutoff point has been selected.
\emph{Clinical Chemistry}, 32, 7.
\doi{10.1093/clinchem/32.7.1341}

Leeflang, MMG., Moons, KGM., Reitsma, JB., & Zwinderman, AH. (2008).
Bias in Sensitivity and Specificity Caused by Data-Driven Selection of Optimal Cutoff Values: Mechanisms, Magnitude, and Solutions.
\emph{Clinical Chemistry}, 54, 4.
\doi{10.1373/clinchem.2007.096032}

Robin, X., Turck, N., Hainard, A., Tiberti, N., Lisacek, F., Sanchez, JC., & Müller, M. (2011).
pROC: an open-source package for R and S+ to analyze and compare ROC curves.
\emph{BMC Bioinformatics}, 12, 77.
\doi{10.1186/1471-2105-12-7}

O'Connell, GC., Alder, ML., Smothers, CG., Still, CH., Webel, AR., & Moore, S.M. (2020).
Diagnosis of ischemic stroke using circulating levels of brain-specific proteins measured via high-sensitivity digital ELISA.
\emph{Brain Research}, 15, 1739.
\doi{10.1016/j.brainres.2020.146861}
}
\seealso{
\ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
\ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
\ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
\ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{pROC::coords()}} \cr
\code{\link[=find.distr]{find.distr()}} \cr
\code{\link[=get.fits]{get.fits()}} \cr
\code{\link[=sm.coords]{sm.coords()}} \cr
\code{\link{blood_data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fits.R
\name{get.fits}
\alias{get.fits}
\title{Return distribution fits for smoothed ROC curves.}
\usage{
get.fits(smooth.roc)
}
\arguments{
\item{smooth.roc}{A \code{smooth.roc} object generated by \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} using \code{method = "fitdistr"}.}
}
\value{
A nested list with the following elements:
\item{\code{controls}}{A list containing fit information for the fitted control distribution:
\describe{
\item{\code{distribution}}{The distribution family.}
\item{\code{parameters}}{A named numeric vector containing the distribution parameters.}
\item{\code{aic}}{The AIC value.}
}
}
\item{\code{cases}}{A list containing fit information for the fitted case distribution:
\describe{
\item{\code{distribution}}{The distribution family.}
\item{\code{parameters}}{A named numeric vector containing the distribution parameters.}
\item{\code{aic}}{The AIC value.}
}
}

Objects returned by \code{\link[=get.fits]{get.fits()}} contain a custom print method that displays a human readable summary of the fitted distributions when printed at the console.
}
\description{
Retrieves the  distribution fits associated with smoothed ROC curves generated using the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package via the\verb{“fitdist”} method. A \code{smooth.roc} object is provided as input, and the fitted case and control distribution families, parameters, and AIC values are returned. Can be used downstream of \code{\link[=find.distr]{find.distr()}} and \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} to inspect and evaluate the final distribution fits ultimately used to generate the smoothed ROC curve.
}
\examples{
# load packages and example blood biomarker data

library(pROC)
library(rocTools)
data(blood_data)

# build raw ROC curve

raw_roc <- roc(
  blood_data,
  response = "Diagnosis",
  predictor = "NfL"
)

# find best fitting case and control distribution
# families to use for ROC smoothing

best_fits <- find.distr(raw_roc)

# use best fitting families to build smooth ROC curve

smooth_roc <- smooth(
  raw_roc,
  method = "fitdistr",
  density.controls = best_fits$density.controls,
  density.cases = best_fits$density.cases,
  start.controls = best_fits$start.controls,
  start.cases = best_fits$start.cases
)

# print final distribution fits used
# to build the smoothed curve

get.fits(smooth_roc)

}
\seealso{
\ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
\ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
\ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
\link{rocTools-package} \cr
\code{\link[=find.distr]{find.distr()}} \cr
\code{\link{blood_data}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blood_data.R
\docType{data}
\name{blood_data}
\alias{blood_data}
\title{Example blood biomarker dataset.}
\format{
A data frame with one column containing stroke diagnoses and another column containing plasma neurofilament-light chain levels in pg/mL.
}
\description{
A real-world dataset containing plasma neurofilament-light chain (NfL) levels measured in 33 neurologically normal controls and 14 patients diagnosed with ischemic stroke as described by O'Connell et al. (2020).
}
\examples{
#load example dataset

data(blood_data)

#return dataset structure

str(blood_data)

}
\references{
O'Connell, GC., Alder, ML., Smothers, CG., Still, CH., Webel, AR., & Moore, S.M. (2020).
Diagnosis of ischemic stroke using circulating levels of brain-specific proteins measured via high-sensitivity digital ELISA.
\emph{Brain Research}, 15, 1739.
\doi{10.1016/j.brainres.2020.146861}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sm_coords.R
\name{sm.coords}
\alias{sm.coords}
\title{Calculate diagnostic performance and cutoff estimates from smoothed ROC curves.}
\usage{
sm.coords(
  smooth.roc,
  type = c("best", "specificity", "sensitivity"),
  target.value = NULL,
  method = c("continuous", "grid"),
  quiet = FALSE
)
}
\arguments{
\item{smooth.roc}{A \code{smooth.roc} object generated by \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} using \code{method = "fitdistr"}.}

\item{type}{Character string specifying the criterion used to define the diagnostic cutoff to return estimates for. If \code{"best"}, estimates for the cutoff that maximizes the Youden index are calculated and returned. If \code{"specificity"}, estimates for the cutoff that achieves a target specificity value specified by \code{target.value} are calculated and returned. If \code{"sensitivity"}, estimates for the cutoff that achieves a target sensitivity value specified by \code{target.value} are calculated and returned.}

\item{target.value}{Numeric value ranging between 0 and 1 that specifies the target sensitivity or specificity value defining the cutoff to calculate estimates for when \code{type} is not \code{"best"}.}

\item{method}{Character string specifying the methodology used to generate estimates. If \code{"continuous"}, values are estimated via continuous optimization of the smoothed ROC curve using the fitted case and control population probability density functions. If \code{"grid"}, cutoff values are selected via optimization over the grid-discretized operating points of the smoothed ROC curve, matching the internal behavior of the native \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{coords()}}  function of the ‘pROC’ package. \code{"continuous"} provides finer resolution and potentially more accurate estimates; however, sensitivity and specificity values will not perfectly match those returned by the \ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{coords()}}  function. \code{"continuous"} is default.}

\item{quiet}{Specifies whether to silence messages generated during estimate calculation. Default value is \code{FALSE}.}
}
\value{
A named numeric vector containing the estimated diagnostic performance and cutoff values:

\item{\code{threshold}}{ The estimated diagnostic cutoff value.}
\item{\code{specificity}}{The estimated specificity at the cutoff value.}
\item{\code{sensitivity}}{The estimated sensitivity at the cutoff value.}
}
\description{
Calculates population-level sensitivity, specificity, and diagnostic cutoff estimates from smoothed ROC curves generated using the \ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{smooth()}} function of the ‘pROC’ package via the \verb{“fitdist”} method. A \code{smooth.roc} object is provided as input, and metrics can be calculated for the estimated diagnostic cutoff that maximizes the Youden index, or for the estimated diagnostic cutoff that achieves a user specified sensitivity or specificity value.
}
\examples{
# load packages and example blood biomarker data

library(pROC)
library(rocTools)
data(blood_data)

# build raw ROC curve

raw_roc <- roc(
  blood_data,
  response = "Diagnosis",
  predictor = "NfL"
)

# find best fitting case and control distribution
# families to use for ROC smoothing

best_fits <- find.distr(raw_roc)

# use best fitting families to build smooth ROC curve

smooth_roc <- smooth(
  raw_roc,
  method = "fitdistr",
  density.controls = best_fits$density.controls,
  density.cases = best_fits$density.cases,
  start.controls = best_fits$start.controls,
  start.cases = best_fits$start.cases
)

# calculate sensitivity, specificity, and cutoff
# estimates for the max Youden cutoff

sm.coords(smooth_roc, type = "best")

# calculate sensitivity, specificity, and cutoff estimates
# for the cutoff that yields 90\% sensitivity

sm.coords(
  smooth_roc,
  type = "sensitivity",
  target.value = 0.9
)

# calculate sensitivity, specificity, and cutoff estimates
# for the cutoff that yields 75\% specificity

sm.coords(
  smooth_roc,
  type = "specificity",
  target.value = 0.75
)

}
\seealso{
\ifelse{html}{\link[pROC]{pROC-package}}{pROC-package} \cr
\ifelse{html}{\code{\link[pROC:roc]{roc()}}}{\code{pROC::roc()}} \cr
\ifelse{html}{\code{\link[pROC:smooth]{smooth()}}}{\code{pROC::smooth()}} \cr
\ifelse{html}{\code{\link[pROC:coords]{coords()}}}{\code{pROC::coords()}} \cr
\link{rocTools-package} \cr
\code{\link[=find.distr]{find.distr()}} \cr
\code{\link{blood_data}}
}
